@@ -39,7 +39,7 @@ patientRoute.post(
     const { models, body: patient } = req;
 
     const potentialDuplicates = await models.Patient.sequelize.query(
-      `SELECT 
+      `SELECT
         p.*,
         reference_data.name AS "villageName"
       FROM find_potential_patient_duplicates(:patient) p
@@ -507,7 +507,14 @@ patientRoute.get(
     const { PatientOngoingPrescription, Prescription } = models;
     const { order = 'ASC', orderBy = 'medication.name', page, rowsPerPage } = query;
 
+    const medicationFilter = {};
+    const canListSensitiveMedication = req.ability.can('list', 'SensitiveMedication');
+    if (!canListSensitiveMedication) {
+      medicationFilter['$medication.referenceDrug.is_sensitive$'] = false;
+    }
+
     const baseQuery = {
+      where: medicationFilter,
       include: [
         ...Prescription.getListReferenceAssociations(),
         {
@@ -515,6 +522,15 @@ patientRoute.get(
           as: 'patientOngoingPrescription',
           where: { patientId },
         },
+        {
+          model: models.ReferenceData,
+          as: 'medication',
+          include: {
+            model: models.ReferenceDrug,
+            as: 'referenceDrug',
+            attributes: ['referenceDataId', 'isSensitive'],
+          },
+        },
       ],
     };
 
@@ -528,9 +544,9 @@ patientRoute.get(
         orderBy === 'route'
           ? [
               literal(
-                `CASE "route" ${Object.entries(DRUG_ROUTE_LABELS)
-                  .map(([value, label]) => `WHEN '${value}' THEN '${label}'`)
-                  .join(' ')} ELSE "route" END`,
+                `CASE "Prescription"."route" ${Object.entries(DRUG_ROUTE_LABELS)
+                  .map(([value, label]) => `WHEN '${value}' THEN '${label.replace(/'/g, "''")}'`)
+                  .join(' ')} ELSE "Prescription"."route" END`,
               ),
               order.toUpperCase(),
             ]
@@ -589,7 +605,14 @@ patientRoute.get(
       return;
     }
 
+    const medicationFilter = {};
+    const canListSensitiveMedication = req.ability.can('list', 'SensitiveMedication');
+    if (!canListSensitiveMedication) {
+      medicationFilter['$medication.referenceDrug.is_sensitive$'] = false;
+    }
+
     const dischargeMedications = await Prescription.findAll({
+      where: medicationFilter,
       include: [
         ...Prescription.getListReferenceAssociations(),
         {
@@ -600,6 +623,15 @@ patientRoute.get(
             encounterId: lastInpatientEncounter.id,
           },
         },
+        {
+          model: req.models.ReferenceData,
+          as: 'medication',
+          include: {
+            model: req.models.ReferenceDrug,
+            as: 'referenceDrug',
+            attributes: ['referenceDataId', 'isSensitive'],
+          },
+        },
       ],
       order: [[...orderBy.split('.'), order.toUpperCase()]],
     });
